@@ -30,13 +30,13 @@
 """Read from and write to tar format archives.
 """
 
-__version__ = "$Revision: 70528 $"
+__version__ = "$Revision: 73770 $"
 # $Source$
 
 version     = "0.9.0"
 __author__  = "Lars Gustbel (lars@gustaebel.de)"
-__date__    = "$Date: 2009-03-22 22:29:48 +0100 (So, 22. Mär 2009) $"
-__cvsid__   = "$Id: tarfile.py 70528 2009-03-22 21:29:48Z lars.gustaebel $"
+__date__    = "$Date: 2009-07-02 11:37:21 -0400 (Thu, 02 Jul 2009) $"
+__cvsid__   = "$Id: tarfile.py 73770 2009-07-02 15:37:21Z jesus.cea $"
 __credits__ = "Gustavo Niemeyer, Niels Gustbel, Richard Townsend."
 
 #---------
@@ -1591,7 +1591,8 @@ class TarFile(object):
         return self.format == USTAR_FORMAT
     def _setposix(self, value):
         import warnings
-        warnings.warn("use the format attribute instead", DeprecationWarning)
+        warnings.warn("use the format attribute instead", DeprecationWarning,
+                      2)
         if value:
             self.format = USTAR_FORMAT
         else:
@@ -1774,7 +1775,7 @@ class TarFile(object):
     def getmember(self, name):
         """Return a TarInfo object for member `name'. If `name' can not be
            found in the archive, KeyError is raised. If a member occurs more
-           than once in the archive, its last occurence is assumed to be the
+           than once in the archive, its last occurrence is assumed to be the
            most up-to-date version.
         """
         tarinfo = self._getmember(name)
