@@ -25,16 +25,10 @@ PyGrammar_AddAccelerators(grammar *g)
 {
 	dfa *d;
 	int i;
-#ifdef Py_DEBUG
-	fprintf(stderr, "Adding parser accelerators ...\n");
-#endif
 	d = g->g_dfa;
 	for (i = g->g_ndfas; --i >= 0; d++)
 		fixdfa(g, d);
 	g->g_accel = 1;
-#ifdef Py_DEBUG
-	fprintf(stderr, "Done.\n");
-#endif
 }
 
 void
@@ -50,7 +44,7 @@ PyGrammar_RemoveAccelerators(grammar *g)
 		s = d->d_state;
 		for (j = 0; j < d->d_nstates; j++, s++) {
 			if (s->s_accel)
-				PyMem_DEL(s->s_accel);
+				PyObject_FREE(s->s_accel);
 			s->s_accel = NULL;
 		}
 	}
@@ -74,7 +68,11 @@ fixstate(grammar *g, state *s)
 	int *accel;
 	int nl = g->g_ll.ll_nlabels;
 	s->s_accept = 0;
-	accel = PyMem_NEW(int, nl);
+	accel = (int *) PyObject_MALLOC(nl * sizeof(int));
+	if (accel == NULL) {
+		fprintf(stderr, "no mem to build parser accelerators\n");
+		exit(1);
+	}
 	for (k = 0; k < nl; k++)
 		accel[k] = -1;
 	a = s->s_arc;
@@ -95,27 +93,10 @@ fixstate(grammar *g, state *s)
 			}
 			for (ibit = 0; ibit < g->g_ll.ll_nlabels; ibit++) {
 				if (testbit(d1->d_first, ibit)) {
-#ifdef applec
-#define MPW_881_BUG			/* Undefine if bug below is fixed */
-#endif
-#ifdef MPW_881_BUG
-					/* In 881 mode MPW 3.1 has a code
-					   generation bug which seems to
-					   set the upper bits; fix this by
-					   explicitly masking them off */
-					int temp;
-#endif
 					if (accel[ibit] != -1)
 						printf("XXX ambiguity!\n");
-#ifdef MPW_881_BUG
-					temp = 0xFFFF &
-						(a->a_arrow | (1 << 7) |
-						 ((type - NT_OFFSET) << 8));
-					accel[ibit] = temp;
-#else
 					accel[ibit] = a->a_arrow | (1 << 7) |
 						((type - NT_OFFSET) << 8);
-#endif
 				}
 			}
 		}
@@ -130,7 +111,7 @@ fixstate(grammar *g, state *s)
 		k++;
 	if (k < nl) {
 		int i;
-		s->s_accel = PyMem_NEW(int, nl-k);
+		s->s_accel = (int *) PyObject_MALLOC((nl-k) * sizeof(int));
 		if (s->s_accel == NULL) {
 			fprintf(stderr, "no mem to add parser accelerators\n");
 			exit(1);
@@ -140,5 +121,5 @@ fixstate(grammar *g, state *s)
 		for (i = 0; k < nl; i++, k++)
 			s->s_accel[i] = accel[k];
 	}
-	PyMem_DEL(accel);
+	PyObject_FREE(accel);
 }
